#ifndef MSP_GRAPHICS_IMAGELOADER_H_
#define MSP_GRAPHICS_IMAGELOADER_H_

#include "image.h"

namespace Msp {
namespace Graphics {

class unsupported_image_format: public std::runtime_error
{
public:
	unsupported_image_format(const std::string &w): std::runtime_error(w) { }
	virtual ~unsupported_image_format() throw() { }
};

class bad_image_data: public std::runtime_error
{
public:
	bad_image_data(const std::string &w): std::runtime_error(w) { }
	virtual ~bad_image_data() throw() { }
};


class ImageLoader
{
protected:
	class RegisterBase
	{
	protected:
		RegisterBase();
	public:
		virtual ~RegisterBase() { }

		virtual unsigned get_signature_size() const = 0;
		virtual bool detect(const std::string &) const = 0;
		virtual ImageLoader *create(IO::Seekable &) const = 0;
	};

private:
	IO::Base *source;

	static std::list<RegisterBase *> &get_registered_loaders();
	static bool registered_loaders_changed;
	static bool signature_size_compare(RegisterBase *, RegisterBase *);

protected:
	ImageLoader();
public:
	virtual ~ImageLoader();

	static ImageLoader *open_file(const std::string &);
	static ImageLoader *open_io(IO::Seekable &);

	virtual void load(Image::Data &) = 0;
};


template<typename T>
class RegisteredImageLoader: public ImageLoader
{
private:
	class Register: public RegisterBase
	{
	public:
		virtual unsigned get_signature_size() const { return T::get_signature_size(); }
		virtual bool detect(const std::string &s) const { return T::detect(s); }
		virtual ImageLoader *create(IO::Seekable &io) const { return new T(io); }
	};

	static Register reg;

protected:
	RegisteredImageLoader() { (void)reg; }
};

template<typename T>
typename RegisteredImageLoader<T>::Register RegisteredImageLoader<T>::reg;

} // namespace Graphics
} // namespace Msp

#endif
